"""
all-geo.py v0.01

Runs tools for n-value, iefa, and obstruction review. Applies appropriate style
and adds layers to current mxd. This must be run from arcgis.

Mike Bannister
mike.bannister@respec.com
2017
"""

# TODO - All Tools: search for missing reaches in GIS vs RAS

import arcpy
import sys
import os
import tempfile
arcpy.AddMessage(sys.version)
path = os.path.join(os.path.dirname(__file__))
sys.path.insert(0, os.path.join(path, 'Block Obs Review'))
sys.path.insert(0, os.path.join(path, 'IEFA Review'))
sys.path.insert(0, os.path.join(path, 'N-value Review'))
arcpy.env.overwriteOutput = True
import blocked_review
import iefa_review
import n_value_review
"""
geofile = arcpy.GetParameterAsText(0)
xs_shape_file = arcpy.GetParameterAsText(1)
xs_id_field = arcpy.GetParameterAsText(2)
river_field = arcpy.GetParameterAsText(3)
reach_field = arcpy.GetParameterAsText(4)
out_dir = arcpy.GetParameterAsText(5)
out_prefix = arcpy.GetParameterAsText(6)
"""
OVER_WRITE = True
currentDir=os.getcwd()
IEFA_STYLE = r"{}\layer_styles\IEFA_Symbology.lyrx".format(currentDir)
OBSTRUCTION_STYLE = r"{}\layer_styles\Blocked_Symbology.lyrx".format(currentDir)
N_VALUE_STYLE = r"{}\layer_styles\NValue_Symbology.lyrx".format(currentDir)


def file_check(outfile):
    """
    Checks if outfile exists and deletes it if OVER_WRITE is true
    """
    if os.path.isfile(outfile):
        if OVER_WRITE:
            arcpy.AddWarning(outfile + ' exists. Deleting.')
            arcpy.Delete_management(outfile)
        else:
            arcpy.AddError(outfile + ' exists and over write is turned off!')
            raise Exception(outfile + ' exists and overwirte is turned off')

def main_geo(geofile,xs_shape_file,xs_id_field,river_field,reach_field,out_dir,out_prefix):
    mxd = arcpy.mp.ArcGISProject("CURRENT")
        
    # Get parameters from Arc
    
    
    # Grab model name from RAS geometry file or use supplied name for shapefiles
    if out_prefix == '':
        geo_name = geofile.split('\\')[-1][:-4]
    else:
        geo_name = out_prefix

    if not os.path.exists(out_dir):
        os.mkdir(out_dir)
    arcpy.AddMessage('\n'+'*'*20+' Creating N-value review lines... '+geo_name)
    
    outfile = os.path.join(out_dir, geo_name + '_N_Value.shp')
    file_check(outfile)
    n_value_review.n_value_review(geofile, xs_shape_file, xs_id_field, river_field, reach_field, outfile)
    nName=geo_name + '_N_Value'
    new_layer = arcpy.management.CreateFeatureclass(out_name=nName,geometry_type='Polyline')

    mxd = arcpy.mp.ArcGISProject("CURRENT")
    ags_map = mxd.listMaps()[0]
    lyr=ags_map.addDataFromPath(outfile) 
    sym = lyr.symbology
    sym.updateRenderer('UniqueValueRenderer')
    sym.renderer.fields = ['Mannings_n']
    for grp in sym.renderer.groups:
        for itm in grp.items:
            #itm.symbol.color = {"RGB": [249, 30, 22, 100]}      # red
            #itm.symbol.outlineColor = {"RGB": [0, 0, 0,75]}     # null colour for symbol outline
            itm.symbol.size = 2

           
    colorRamp = mxd.listColorRamps("Basic Random")[0]
    sym.renderer.colorRamp = colorRamp

    lyr.symbology = sym
    #if lyr.supports("LABELCLASSES"):
        #print lyr.name + " supports label classes"
    lblClass = lyr.listLabelClasses()[0]
    lblClass.expression = "\"<FNT name = 'Arial' size = '8'>\" + \"<CLR red = '0' green = '0' blue = '0'>\"  +  round($feature.Mannings_n,4)  +  \"</CLR>\" +  \"</FNT>\""
    lyr.showLabels = True
   
    
    arcpy.AddMessage('\n'+'*'*20+' Creating IEFA review lines... ')
    outfile = os.path.join(out_dir, geo_name + '_IEFA.shp')
    file_check(outfile)
    iefa_review.iefa_review(geofile, xs_shape_file, xs_id_field, river_field, reach_field, outfile)
    iefaName=geo_name + '_IEFA'
    new_layer = arcpy.management.CreateFeatureclass(out_name=iefaName,geometry_type='Polyline')
    arcpy.management.MakeFeatureLayer(new_layer,iefaName)
    
    
    old_lyr = ags_map.addDataFromPath(outfile)    
    arcpy.management.ApplySymbologyFromLayer(old_lyr, IEFA_STYLE , "VALUE_FIELD IEFA OBJECTID", "UPDATE")
    new_lyr_file = arcpy.mp.LayerFile(IEFA_STYLE )
    new_lyr = new_lyr_file.listLayers()[0]
    old_lyr_name = old_lyr.name
    new_lyr.updateConnectionProperties(new_lyr.connectionProperties, old_lyr.connectionProperties)
    new_lyr.name = old_lyr_name
    new_lyr_file.save()
    ags_map.insertLayer(old_lyr, new_lyr_file)
    ags_map.removeLayer(old_lyr)

    
    arcpy.AddMessage('\n'+'*'*20+' Creating obstruction review lines... ')
    outfile = os.path.join(out_dir, geo_name + '_Blocked.shp')
    file_check(outfile)
    blocked_review.obstruction_review(geofile, xs_shape_file, xs_id_field, river_field, reach_field, outfile)
    BobsName=geo_name + '_Blocked'
    new_layer = arcpy.management.CreateFeatureclass(out_name=BobsName,geometry_type='Polyline')
    
    old_lyr = ags_map.addDataFromPath(outfile)    
    arcpy.management.ApplySymbologyFromLayer(old_lyr, OBSTRUCTION_STYLE, "VALUE_FIELD Blocked Blocked", "UPDATE")
    new_lyr_file = arcpy.mp.LayerFile(OBSTRUCTION_STYLE )
    new_lyr = new_lyr_file.listLayers()[0]
    old_lyr_name = old_lyr.name
    new_lyr.updateConnectionProperties(new_lyr.connectionProperties, old_lyr.connectionProperties)
    new_lyr.name = old_lyr_name
    new_lyr_file.save()
    ags_map.insertLayer(old_lyr, new_lyr_file)
    ags_map.removeLayer(old_lyr)

if __name__ == '__main__':
    main_geo(geofile,xs_shape_file,xs_id_field,river_field,reach_field,out_dir,out_prefix)
        
